package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1Encodable;
import cn.com.infosec.asn1.ASN1EncodableVector;
import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.ASN1Set;
import cn.com.infosec.asn1.DEREncodable;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DERObject;
import cn.com.infosec.asn1.DEROutputStream;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.asn1.DERSet;
import cn.com.infosec.asn1.pkcs.ContentInfo;
import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.asn1.pkcs.SignedData;
import cn.com.infosec.openssl.PEMWriter;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.security.NoSuchProviderException;
import java.security.cert.CertPath;
import java.security.cert.Certificate;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import javax.security.auth.x500.X500Principal;

public class PKIXCertPath extends CertPath {
   static final List certPathEncodings;
   private List certificates;

   static {
      ArrayList var10000 = new ArrayList();
      var10000.add("PkiPath");
      var10000.add("PEM");
      var10000.add("PKCS7");
      certPathEncodings = Collections.unmodifiableList(var10000);
   }

   private List sortCerts(List certs) {
      if (certs.size() < 2) {
         return certs;
      } else {
         X500Principal var8 = ((X509Certificate)certs.get(0)).getIssuerX500Principal();
         boolean var2 = true;

         int var3;
         for(var3 = 1; var3 != certs.size(); ++var3) {
            if (!var8.equals(((X509Certificate)certs.get(var3)).getSubjectX500Principal())) {
               var2 = false;
               break;
            }

            var8 = ((X509Certificate)certs.get(var3)).getIssuerX500Principal();
         }

         if (var2) {
            return certs;
         } else {
            ArrayList var9;
            var9 = new ArrayList.<init>(certs.size());
            ArrayList var10;
            var10 = new ArrayList.<init>(certs);

            for(var3 = 0; var3 < certs.size(); ++var3) {
               X509Certificate var4;
               X509Certificate var10000 = var4 = (X509Certificate)certs.get(var3);
               boolean var5 = false;
               X500Principal var6 = var10000.getSubjectX500Principal();

               for(int var7 = 0; var7 != certs.size(); ++var7) {
                  if (((X509Certificate)certs.get(var7)).getIssuerX500Principal().equals(var6)) {
                     var5 = true;
                     break;
                  }
               }

               if (!var5) {
                  var9.add(var4);
                  certs.remove(var3);
               }
            }

            if (var9.size() > 1) {
               return var10;
            } else {
               byte var11 = 0;

               for(int var12 = var11; var12 != var9.size(); var12 = var3) {
                  X500Principal var13 = ((X509Certificate)var9.get(var12)).getIssuerX500Principal();

                  for(var3 = 0; var3 < certs.size(); ++var3) {
                     X509Certificate var14;
                     if (var13.equals((var14 = (X509Certificate)certs.get(var3)).getSubjectX500Principal())) {
                        var9.add(var14);
                        certs.remove(var3);
                        break;
                     }
                  }

                  var3 = var12 + 1;
               }

               if (certs.size() > 0) {
                  return var10;
               } else {
                  return var9;
               }
            }
         }
      }
   }

   PKIXCertPath(List certificates) {
      super("X.509");
      this.certificates = this.sortCerts(new ArrayList(certificates));
   }

   PKIXCertPath(InputStream inStream, String encoding) {
      String var10000 = encoding;
      super("X.509");

      label198: {
         IOException var55;
         label197: {
            NoSuchProviderException var54;
            label196: {
               boolean var10001;
               boolean var56;
               try {
                  var56 = var10000.equalsIgnoreCase("PkiPath");
               } catch (IOException var43) {
                  var55 = var43;
                  var10001 = false;
                  break label197;
               } catch (NoSuchProviderException var44) {
                  var54 = var44;
                  var10001 = false;
                  break label196;
               }

               PKIXCertPath var62;
               CertificateFactory var63;
               if (var56) {
                  label202: {
                     DERObject var57;
                     try {
                        var57 = (new ASN1InputStream(inStream)).readObject();
                     } catch (IOException var25) {
                        var55 = var25;
                        var10001 = false;
                        break label197;
                     } catch (NoSuchProviderException var26) {
                        var54 = var26;
                        var10001 = false;
                        break label202;
                     }

                     DERObject inStream1 = var57;

                     try {
                        var56 = var57 instanceof ASN1Sequence;
                     } catch (IOException var23) {
                        var55 = var23;
                        var10001 = false;
                        break label197;
                     } catch (NoSuchProviderException var24) {
                        var54 = var24;
                        var10001 = false;
                        break label202;
                     }

                     if (var56) {
                        label203: {
                           Enumeration var61;
                           try {
                              var62 = this;
                              var61 = ((ASN1Sequence)inStream1).getObjects();
                           } catch (IOException var19) {
                              var55 = var19;
                              var10001 = false;
                              break label197;
                           } catch (NoSuchProviderException var20) {
                              var54 = var20;
                              var10001 = false;
                              break label203;
                           }

                           Enumeration inStream2 = var61;

                           try {
                              var62.certificates = new ArrayList();
                              var63 = CertificateFactory.getInstance("X.509", "INFOSEC");
                           } catch (IOException var17) {
                              var55 = var17;
                              var10001 = false;
                              break label197;
                           } catch (NoSuchProviderException var18) {
                              var54 = var18;
                              var10001 = false;
                              break label203;
                           }

                           CertificateFactory encoding1 = var63;

                           while(true) {
                              try {
                                 var56 = inStream2.hasMoreElements();
                              } catch (IOException var15) {
                                 var55 = var15;
                                 var10001 = false;
                                 break label197;
                              } catch (NoSuchProviderException var16) {
                                 var54 = var16;
                                 var10001 = false;
                                 break;
                              }

                              if (!var56) {
                                 break label198;
                              }

                              ByteArrayOutputStream var64;
                              try {
                                 var62 = this;
                                 var64 = new ByteArrayOutputStream;
                              } catch (IOException var13) {
                                 var55 = var13;
                                 var10001 = false;
                                 break label197;
                              } catch (NoSuchProviderException var14) {
                                 var54 = var14;
                                 var10001 = false;
                                 break;
                              }

                              ByteArrayOutputStream var3 = var64;

                              DEROutputStream var65;
                              try {
                                 var64.<init>();
                                 var65 = new DEROutputStream;
                              } catch (IOException var11) {
                                 var55 = var11;
                                 var10001 = false;
                                 break label197;
                              } catch (NoSuchProviderException var12) {
                                 var54 = var12;
                                 var10001 = false;
                                 break;
                              }

                              DEROutputStream var10002 = var65;
                              DEROutputStream var10003 = var65;

                              ByteArrayInputStream var66;
                              try {
                                 var10003.<init>(var3);
                                 var10002.writeObject(inStream2.nextElement());
                                 var65.close();
                                 var66 = new ByteArrayInputStream;
                              } catch (IOException var9) {
                                 var55 = var9;
                                 var10001 = false;
                                 break label197;
                              } catch (NoSuchProviderException var10) {
                                 var54 = var10;
                                 var10001 = false;
                                 break;
                              }

                              ByteArrayInputStream var4 = var66;

                              List var68;
                              try {
                                 var66.<init>(var3.toByteArray());
                                 var68 = var62.certificates;
                              } catch (IOException var7) {
                                 var55 = var7;
                                 var10001 = false;
                                 break label197;
                              } catch (NoSuchProviderException var8) {
                                 var54 = var8;
                                 var10001 = false;
                                 break;
                              }

                              CertificateFactory var67 = encoding1;
                              ByteArrayInputStream var58 = var4;
                              byte var52 = 0;

                              try {
                                 var68.add(var52, var67.generateCertificate(var58));
                              } catch (IOException var5) {
                                 var55 = var5;
                                 var10001 = false;
                                 break label197;
                              } catch (NoSuchProviderException var6) {
                                 var54 = var6;
                                 var10001 = false;
                                 break;
                              }
                           }
                        }
                     } else {
                        try {
                           throw new CertificateException("input stream does not contain a ASN1 SEQUENCE while reading PkiPath encoded data to load CertPath");
                        } catch (IOException var21) {
                           var55 = var21;
                           var10001 = false;
                           break label197;
                        } catch (NoSuchProviderException var22) {
                           var54 = var22;
                           var10001 = false;
                        }
                     }
                  }
               } else {
                  label205: {
                     try {
                        var56 = encoding.equalsIgnoreCase("PKCS7");
                     } catch (IOException var41) {
                        var55 = var41;
                        var10001 = false;
                        break label197;
                     } catch (NoSuchProviderException var42) {
                        var54 = var42;
                        var10001 = false;
                        break label205;
                     }

                     if (!var56) {
                        try {
                           var56 = encoding.equalsIgnoreCase("PEM");
                        } catch (IOException var39) {
                           var55 = var39;
                           var10001 = false;
                           break label197;
                        } catch (NoSuchProviderException var40) {
                           var54 = var40;
                           var10001 = false;
                           break label205;
                        }

                        if (!var56) {
                           StringBuffer var59;
                           CertificateException var70;
                           CertificateException var72;
                           try {
                              var72 = new CertificateException;
                              var70 = var72;
                              var59 = new StringBuffer;
                           } catch (IOException var29) {
                              var55 = var29;
                              var10001 = false;
                              break label197;
                           } catch (NoSuchProviderException var30) {
                              var54 = var30;
                              var10001 = false;
                              break label205;
                           }

                           StringBuffer var60 = var59;

                           try {
                              var60.<init>("unsupported encoding: ");
                              var70.<init>(var59.append(encoding).toString());
                              throw var72;
                           } catch (IOException var27) {
                              var55 = var27;
                              var10001 = false;
                              break label197;
                           } catch (NoSuchProviderException var28) {
                              var54 = var28;
                              var10001 = false;
                              break label205;
                           }
                        }
                     }

                     BufferedInputStream var69;
                     try {
                        var62 = this;
                        var69 = new BufferedInputStream;
                     } catch (IOException var37) {
                        var55 = var37;
                        var10001 = false;
                        break label197;
                     } catch (NoSuchProviderException var38) {
                        var54 = var38;
                        var10001 = false;
                        break label205;
                     }

                     BufferedInputStream encoding2 = var69;

                     try {
                        var69.<init>(inStream);
                        var62.certificates = new ArrayList();
                        var63 = CertificateFactory.getInstance("X.509", "INFOSEC");
                     } catch (IOException var35) {
                        var55 = var35;
                        var10001 = false;
                        break label197;
                     } catch (NoSuchProviderException var36) {
                        var54 = var36;
                        var10001 = false;
                        break label205;
                     }

                     CertificateFactory inStream3 = var63;

                     while(true) {
                        Certificate var71;
                        try {
                           var71 = inStream3.generateCertificate(encoding2);
                        } catch (IOException var33) {
                           var55 = var33;
                           var10001 = false;
                           break label197;
                        } catch (NoSuchProviderException var34) {
                           var54 = var34;
                           var10001 = false;
                           break;
                        }

                        Certificate var53 = var71;
                        if (var71 == null) {
                           break label198;
                        }

                        try {
                           this.certificates.add(var53);
                        } catch (IOException var31) {
                           var55 = var31;
                           var10001 = false;
                           break label197;
                        } catch (NoSuchProviderException var32) {
                           var54 = var32;
                           var10001 = false;
                           break;
                        }
                     }
                  }
               }
            }

            NoSuchProviderException var45 = var54;
            throw new CertificateException("Infosec provider not found while trying to get a CertificateFactory:\n" + var45.toString());
         }

         IOException var46 = var55;
         throw new CertificateException("IOException throw while decoding CertPath:\n" + var46.toString());
      }

      this.certificates = this.sortCerts(this.certificates);
   }

   private DERObject toASN1Object(X509Certificate cert) {
      try {
         return (new ASN1InputStream(cert.getEncoded())).readObject();
      } catch (Exception var2) {
         throw new CertificateEncodingException("Exception while encoding certificate: " + var2.toString());
      }
   }

   private byte[] toDEREncoded(ASN1Encodable obj) {
      IOException var10000;
      label29: {
         ByteArrayOutputStream var7;
         boolean var10001;
         try {
            var7 = new ByteArrayOutputStream;
         } catch (IOException var4) {
            var10000 = var4;
            var10001 = false;
            break label29;
         }

         ByteArrayOutputStream var5;
         ByteArrayOutputStream var8 = var5 = var7;

         DEROutputStream var9;
         try {
            var8.<init>();
            var9 = new DEROutputStream;
         } catch (IOException var3) {
            var10000 = var3;
            var10001 = false;
            break label29;
         }

         DEROutputStream var10002 = var9;
         DEROutputStream var10003 = var9;

         try {
            var10003.<init>(var5);
            var10002.writeObject(obj);
            var9.close();
            return var7.toByteArray();
         } catch (IOException var2) {
            var10000 = var2;
            var10001 = false;
         }
      }

      IOException var6 = var10000;
      throw new CertificateEncodingException("Exeption thrown: " + var6);
   }

   public Iterator getEncodings() {
      return certPathEncodings.iterator();
   }

   public byte[] getEncoded() {
      Iterator var1;
      Object var2;
      return (var1 = this.getEncodings()).hasNext() && (var2 = var1.next()) instanceof String ? this.getEncoded((String)var2) : null;
   }

   public byte[] getEncoded(String encoding) {
      if (encoding.equalsIgnoreCase("PkiPath")) {
         ASN1EncodableVector var13;
         var13 = new ASN1EncodableVector.<init>();
         List var19 = this.certificates;
         ListIterator var16 = var19.listIterator(var19.size());

         while(var16.hasPrevious()) {
            var13.add(this.toASN1Object((X509Certificate)var16.previous()));
         }

         return this.toDEREncoded(new DERSequence(var13));
      } else if (encoding.equalsIgnoreCase("PKCS7")) {
         ContentInfo var11;
         var11 = new ContentInfo.<init>(PKCSObjectIdentifiers.data, (DEREncodable)null);
         ASN1EncodableVector var14;
         var14 = new ASN1EncodableVector.<init>();

         for(int encoding2 = 0; encoding2 != this.certificates.size(); ++encoding2) {
            var14.add(this.toASN1Object((X509Certificate)this.certificates.get(encoding2)));
         }

         SignedData var8;
         SignedData var21 = var8 = new SignedData;
         DERInteger var10002 = new DERInteger(1);
         DERSet encoding3;
         encoding3 = new DERSet.<init>();
         DERSet var4;
         var4 = new DERSet.<init>(var14);
         DERSet var15;
         var15 = new DERSet.<init>();
         var21.<init>(var10002, encoding3, var11, var4, (ASN1Set)null, var15);
         return this.toDEREncoded(new ContentInfo(PKCSObjectIdentifiers.signedData, var8));
      } else if (!encoding.equalsIgnoreCase("PEM")) {
         throw new CertificateEncodingException("unsupported encoding: " + encoding);
      } else {
         ByteArrayOutputStream encoding1;
         encoding1 = new ByteArrayOutputStream.<init>();
         PEMWriter var2;
         var2 = new PEMWriter.<init>(new OutputStreamWriter(encoding1));
         int var3 = 0;

         while(true) {
            int var10000;
            boolean var10001;
            int var20;
            try {
               var10000 = var3;
               var20 = this.certificates.size();
            } catch (Exception var7) {
               var10001 = false;
               break;
            }

            if (var10000 == var20) {
               try {
                  ByteArrayOutputStream var17 = encoding1;
                  var2.close();
                  return var17.toByteArray();
               } catch (Exception var5) {
                  var10001 = false;
                  break;
               }
            } else {
               try {
                  var2.writeObject(this.certificates.get(var3));
               } catch (Exception var6) {
                  var10001 = false;
                  break;
               }

               ++var3;
            }
         }

         throw new CertificateEncodingException("can't encode certificate for PEM encoded path");
      }
   }

   public List getCertificates() {
      return Collections.unmodifiableList(new ArrayList(this.certificates));
   }
}
